/*
    Copyright (C) 2008    Dmitry Ivanov <vonami@gmail.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "abstractproxyfeed.h"
#include "abstractproxyfeed_p.h"

#include "itemlistjob.h"

#include <KIcon>
#include <KDebug>

#include <QtCore/QTimer>

#include <boost/bind.hpp>

#include <algorithm>
#include <numeric>

using namespace boost;
using namespace KRss;

AbstractProxyFeed::AbstractProxyFeed( QObject *parent )
    : Feed( parent ), d( new AbstractProxyFeedPrivate() )
{
}

AbstractProxyFeed::~AbstractProxyFeed()
{
    delete d;
}

Feed::Id AbstractProxyFeed::id() const
{
    return -1;
}

QString AbstractProxyFeed::title() const
{
    return d->m_title;
}

void AbstractProxyFeed::setTitle( const QString &title )
{
    d->m_title = title;
    emit changed( this );
}

QString AbstractProxyFeed::description() const
{
    return d->m_description;
}

void AbstractProxyFeed::setDescription( const QString &description )
{
    d->m_description = description;
    emit changed( this );
}

QString AbstractProxyFeed::type() const
{
    return d->m_type;
}

void AbstractProxyFeed::setType( const QString &type )
{
    d->m_type = type;
    emit changed( this );
}

KIcon AbstractProxyFeed::icon() const
{
    return d->m_icon;
}

void AbstractProxyFeed::setIcon( const KIcon &icon )
{
    d->m_icon = icon;
    emit changed( this );
}

QList<TagId> AbstractProxyFeed::tags() const
{
    return d->m_tags;
}

void AbstractProxyFeed::setTags( const QList<TagId> &tags )
{
    d->m_tags = tags;
    emit changed( this );
}

void AbstractProxyFeed::addTag( const TagId &tag )
{
    if ( !d->m_tags.contains( tag ) ) {
        d->m_tags.append( tag );
        emit changed( this );
    }
}

void AbstractProxyFeed::removeTag( const TagId &tag )
{
    d->m_tags.removeOne( tag );
    emit changed( this );
}

int AbstractProxyFeed::total() const
{
    int sum = 0;
    Q_FOREACH( Feed* const i, d->m_sourceFeeds )
        sum +=i->total();
    return sum;
}

int AbstractProxyFeed::unread() const
{
    int sum = 0;
    Q_FOREACH( Feed* const i, d->m_sourceFeeds )
        sum +=i->unread();
    return sum;
}

int AbstractProxyFeed::fetchInterval() const
{
    // in minutes
    return ( d->m_fetchTimer->interval() / ( 1000 * 60 ) );
}

void AbstractProxyFeed::setFetchInterval( int minutes )
{
    if ( d->m_fetchTimer && d->m_fetchTimer->isActive() ) {
        d->m_fetchTimer->stop();
    }
    else {
        d->m_fetchTimer = new QTimer( this );
        connect( d->m_fetchTimer, SIGNAL( timeout() ), this, SLOT( fetch() ) );
    }

    if ( minutes > 0 ) {
        d->m_fetchTimer->setInterval( minutes * 1000 * 60 );
        d->m_fetchTimer->start();
    }
}

int AbstractProxyFeed::maxArticleNumber() const
{
    return 0;
}

void AbstractProxyFeed::setMaxArticleNumber( int number )
{
    Q_UNUSED( number )
}

int AbstractProxyFeed::maxArticleAge() const
{
    return 0;
}

void AbstractProxyFeed::setMaxArticleAge( int age )
{
    Q_UNUSED( age )
}

ItemListJob* AbstractProxyFeed::createListJob() {
    QList<ItemListJob*> list;
    CompositeItemListJob* job = new CompositeItemListJob( this );
    job->setFilter( filterFunction() );
    Q_FOREACH( Feed* const i, d->m_sourceFeeds )
        job->addSubJob( i->createListJob() );
    return job;
}


void AbstractProxyFeed::fetch() const
{
    Q_FOREACH( Feed *feed, d->m_sourceFeeds ) {
        feed->fetch();
    }
}

void AbstractProxyFeed::abortFetch() const
{
    Q_FOREACH( Feed *feed, d->m_sourceFeeds ) {
        feed->abortFetch();
    }
}

void AbstractProxyFeed::slotFeedFetchStarted( Feed *feed )
{
    Q_FOREACH( AbstractProxyFeedPrivate::Status status, d->m_statuses ) {
        // if I am not the first one, don't emit fetchStarted()
        if ( status == AbstractProxyFeedPrivate::FetchStarted || status == AbstractProxyFeedPrivate::FetchFinished ) {
            d->m_statuses[ feed ] = AbstractProxyFeedPrivate::FetchStarted;
            d->m_percents[ feed ] = 0;
            return;
        }
    }

    d->m_statuses[ feed ] = AbstractProxyFeedPrivate::FetchStarted;
    d->m_percents[ feed ] = 0;
    emit fetchStarted( this );
}

void AbstractProxyFeed::slotFeedFetchPercent( Feed *feed, uint percentage )
{
    d->m_percents[ feed ] = percentage;

    uint total = 0;
    Q_FOREACH( uint percent, d->m_percents ) {
        total += percent / d->m_percents.count();
    }
    emit fetchPercent( this, total );
}

void AbstractProxyFeed::slotFeedFetchFinished( Feed *feed )
{
    d->m_statuses[ feed ] = AbstractProxyFeedPrivate::FetchFinished;
    d->m_percents[ feed ] = 100;

    uint total = 0;
    Q_FOREACH( uint percent, d->m_percents ) {
        total += percent / d->m_percents.count();
    }
    emit fetchPercent( this, total );

    // emit fetchFinished() only if I am the last one
    Q_FOREACH( AbstractProxyFeedPrivate::Status status, d->m_statuses ) {
        if ( status != AbstractProxyFeedPrivate::FetchFinished )
            return;
    }

    emit fetchFinished( this );
}

void AbstractProxyFeed::slotFeedFetchFailed( Feed *feed, const QString &errorMessage )
{
    Q_UNUSED( feed )
    emit fetchFailed( this, errorMessage );
}

void AbstractProxyFeed::slotFeedChanged( Feed *feed )
{
    Q_UNUSED( feed )
    emit changed( this );
}

void AbstractProxyFeed::slotFeedRemoved( Feed *feed )
{
    removeSourceFeed( feed );
}

void AbstractProxyFeed::slotFeedItemsAdded( Feed *feed, const QList<Item> &items_ )
{
    QList<Item> items = items_;

    items.erase( std::remove_if( items.begin(), items.end(), !bind( filterFunction(), _1 ) ),
                 items.end() );

    if ( !items.isEmpty() )
        emit itemsAdded( this, items );
}

void AbstractProxyFeed::slotFeedItemsChanged( Feed *feed, const QList<Item> &items )
{
#if 0 // reimplement when creating ItemCollection or similar (where we know the items we already saw)
    QList<Item> addedItems;
    QList<Item> changedItems;
    QList<Item> removedItems;

    Q_FOREACH( const Item &item, items ) {

        if ( acceptsItem( item ) ) {
            if ( d->m_allItems.contains( item.id() ) ) {
                // item changed
                d->m_allItems[ item.id() ] = item;
                changedItems.append( item );
            }
            else {
                // item added
                d->m_allItems[ item.id() ] = item;
                d->m_feedItems[ feed ].append( item.id() );
                addedItems.append( item );
            }
        }
        else {
            if ( d->m_allItems.contains( item.id() ) ) {
                // item removed
                d->m_allItems.remove( item.id() );
                d->m_feedItems[ feed ].removeOne( item.id() );
                removedItems.append( item );
            }
            else {
                // do nothing
            }
        }
    }

    if ( !addedItems.isEmpty() )
        emit itemsAdded( this, addedItems );

    if ( !changedItems.isEmpty() )
        emit itemsChanged( this, changedItems );

    if ( !removedItems.isEmpty() )
        emit itemsRemoved( this, removedItems );
#endif
}

void AbstractProxyFeed::slotFeedItemsRemoved( Feed *feed, const QList<Item> &items )
{
    emit itemsRemoved( this, items );
}

void AbstractProxyFeed::addSourceFeed( Feed *feed )
{
    // avoid circular dependencies and duplicates
    if ( feed == this || d->m_sourceFeeds.contains( feed ) || !feed ) {
        if ( !feed )
            kWarning() << "Refusing to add null feed";
        else
            kWarning() << "Refusing to add feed:" << feed->id() << ", title:" << feed->title();

        return;
    }

    kDebug() << "Adding new feed at" << d->m_title << ":" << feed->id() << ", title:" << feed->title();
    d->m_sourceFeeds.append( feed );
    d->m_percents[ feed ] = 0;
    d->m_statuses[ feed ] = AbstractProxyFeedPrivate::Idle;

    connect( feed, SIGNAL( fetchStarted( KRss::Feed* ) ), this, SLOT( slotFeedFetchStarted( KRss::Feed* ) ) );
    connect( feed, SIGNAL( fetchPercent( KRss::Feed*, uint ) ), this, SLOT( slotFeedFetchPercent( KRss::Feed*, uint ) ) );
    connect( feed, SIGNAL( fetchFinished( KRss::Feed* ) ), this, SLOT( slotFeedFetchFinished( KRss::Feed* ) ) );
    connect( feed, SIGNAL( fetchFailed( KRss::Feed*, const QString& ) ),
             this, SLOT( slotFeedFetchFailed( KRss::Feed*, const QString& ) ) );

    connect( feed, SIGNAL( changed( KRss::Feed* ) ), this, SLOT( slotFeedChanged( KRss::Feed* ) ) );
    connect( feed, SIGNAL( removed( KRss::Feed* ) ), this, SLOT( slotFeedRemoved( KRss::Feed* ) ) );

    connect( feed, SIGNAL( itemsAdded( KRss::Feed*, const QList<KRss::Item>& ) ),
                this, SLOT( slotFeedItemsAdded( KRss::Feed*, const QList<KRss::Item>& ) ) );
    connect( feed, SIGNAL( itemsChanged( KRss::Feed*, const QList<KRss::Item>& ) ),
                this, SLOT( slotFeedItemsChanged( KRss::Feed*, const QList<KRss::Item>& ) ) );
    connect( feed, SIGNAL( itemsRemoved( KRss::Feed*, const QList<KRss::Item>& ) ),
                this, SLOT( slotFeedItemsRemoved( KRss::Feed*, const QList<KRss::Item>& ) ) );
}

void AbstractProxyFeed::removeSourceFeed( Feed *feed )
{
    if ( !d->m_sourceFeeds.contains( feed ) )
        return;

    //TODO(frank) notify that feed (and items) was removed

    feed->disconnect( this );
    d->m_sourceFeeds.removeOne( feed );
    d->m_percents.remove( feed );
    d->m_statuses.remove( feed );
}

void AbstractProxyFeed::clearAllSourceFeeds()
{
    //TODO(frank) notify that all items are gone

    Q_FOREACH( Feed *feed, d->m_sourceFeeds )
        feed->disconnect( this );

    d->m_sourceFeeds.clear();
    d->m_percents.clear();
    d->m_statuses.clear();
}

QList<Feed*> AbstractProxyFeed::sourceFeeds() const
{
    return d->m_sourceFeeds;
}

#include "abstractproxyfeed.moc"
