/*
    Copyright (C) 2008,2009    Dmitry Ivanov <vonami@gmail.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "persistentfeeddeletejob.h"
#include "persistentfeed.h"
#include "persistentfeed_p.h"
#include "tagprovider.h"
#include "tagjobs.h"

#include <akonadi/collectiondeletejob.h>
#include <KLocale>
#include <KDebug>

using namespace KRss;
using Akonadi::CollectionDeleteJob;

namespace KRss {

class PersistentFeedDeleteJobPrivate
{
public:

    explicit PersistentFeedDeleteJobPrivate( const PersistentFeed *feed, PersistentFeedDeleteJob *qq )
        : q( qq ), m_feed( feed )
    {
    }

    void doStart();
    void slotTagProviderRetrieved( KJob *job );
    void slotReferencesDeleted( KJob *job );
    void slotFeedDeleted( KJob *job );

    PersistentFeedDeleteJob * const q;
    const PersistentFeed *m_feed;
};

} // namespace KRss

void PersistentFeedDeleteJobPrivate::doStart()
{
    TagProviderRetrieveJob *tjob = new TagProviderRetrieveJob();
    QObject::connect( tjob, SIGNAL( result( KJob* ) ), q, SLOT( slotTagProviderRetrieved( KJob* ) ) );
    tjob->start();
}

void PersistentFeedDeleteJobPrivate::slotTagProviderRetrieved( KJob *job )
{
    if ( job->error() ) {
        q->setError( PersistentFeedDeleteJob::CouldNotRetrieveTagProvider );
        q->setErrorText( job->errorString() );
        kWarning() << job->errorString();
        q->emitResult();
        return;
    }

    const TagProviderRetrieveJob * const tjob = qobject_cast<const TagProviderRetrieveJob*>( job );
    Q_ASSERT( tjob );
    const TagProvider *tp = tjob->tagProvider();

    TagDeleteReferencesJob *djob = tp->tagDeleteReferencesJob();
    djob->setReferrer( m_feed );
    QObject::connect( djob, SIGNAL( result( KJob* ) ), q, SLOT( slotReferencesDeleted( KJob* ) ) );
    djob->start();
}

void PersistentFeedDeleteJobPrivate::slotReferencesDeleted( KJob *job )
{
    if ( job->error() ) {
        q->setError( PersistentFeedDeleteJob::CouldNotDeleteReferences );
        q->setErrorText( job->errorString() );
        kWarning() << job->errorString();
        q->emitResult();
        return;
    }

    CollectionDeleteJob *djob = new CollectionDeleteJob( m_feed->d->m_feedCollection );
    QObject::connect( djob, SIGNAL( result( KJob* ) ), q, SLOT( slotFeedDeleted( KJob* ) ) );
    djob->start();
}

void PersistentFeedDeleteJobPrivate::slotFeedDeleted( KJob *job )
{
    if ( job->error() ) {
        q->setError( PersistentFeedDeleteJob::CouldNotDeleteFeed );
        q->setErrorText( job->errorString() );
        kWarning() << job->errorString();
        q->emitResult();
        return;
    }

    q->emitResult();
}


PersistentFeedDeleteJob::PersistentFeedDeleteJob( const PersistentFeed *feed, QObject *parent ) :
    KJob( parent ), d( new PersistentFeedDeleteJobPrivate( feed, this ) )
{
}

PersistentFeedDeleteJob::~PersistentFeedDeleteJob()
{
    delete d;
}

void PersistentFeedDeleteJob::start()
{
    QMetaObject::invokeMethod( this, "doStart", Qt::QueuedConnection );
}

QString PersistentFeedDeleteJob::errorString() const
{
    QString result;
    switch ( error() ) {
        case KJob::NoError:
            result = i18n( "No error." );
            break;
        case PersistentFeedDeleteJob::CouldNotDeleteFeed:
            result = i18n( "Couldn't delete feed.\n%1", errorText() );
            break;
        case PersistentFeedDeleteJob::CouldNotRetrieveTagProvider:
            result = i18n( "Couldn't retrieve the tag provider.\n%1", errorText() );
            break;
        case PersistentFeedDeleteJob::CouldNotDeleteReferences:
            result = i18n( "Couldn't delete the references inside the tag provider.\n%1", errorText() );
            break;
        default:
            result = i18n( "Unknown result code." );
            break;
    }
    return result;
}

#include "persistentfeeddeletejob.moc"
