/*
    Copyright (C) 2008    Dmitry Ivanov <vonami@gmail.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "addfeedjob.h"
#include "libkrss/feedcollection.h"

#include <akonadi/collectioncreatejob.h>
#include <KUrl>
#include <KDebug>

#include <QTimer>

using namespace KRss;

AddFeedJob::AddFeedJob( const FeedCollection &feed, QObject *parent ) :
        KJob( parent ), m_feed( feed ), m_fetchTries( 0 )
{
}

AddFeedJob::~AddFeedJob()
{
}

void AddFeedJob::setReplyMessage( const QDBusMessage &replyMessage )
{
        m_replyMessage = replyMessage;
}

QDBusMessage AddFeedJob::replyMessage() const
{
        return m_replyMessage;
}

FeedCollection AddFeedJob::feed() const
{
        return m_feed;
}

void AddFeedJob::start()
{
        QTimer::singleShot( 0, this, SLOT( doStart() ) );
}

QString AddFeedJob::errorString() const
{
        switch ( error() ) {
                case CollectionCreationFailed:
                        return errorText();
                default:
                        return KJob::errorString();
        }
}

void AddFeedJob::doStart()
{
        Syndication::Loader *loader = Syndication::Loader::create( this, SLOT( fetchDone( Syndication::Loader*,
                                                                                          Syndication::FeedPtr,
                                                                                          Syndication::ErrorCode ) ) );

        kDebug() << "Url:" << m_feed.xmlUrl();
        loader->loadFrom( KUrl( m_feed.xmlUrl() ) );
}

void AddFeedJob::fetchDone( Syndication::Loader *loader, Syndication::FeedPtr feedptr, Syndication::ErrorCode status )
{
        kDebug() << "Status:" << status;

        if ( ( status == Syndication::InvalidXml ) && (m_fetchTries < 3 ) && ( loader->discoveredFeedURL().isValid() ) ) {
                QString newUrl = loader->discoveredFeedURL().url();
                kDebug() << "Refetching by discovered feed url:" << newUrl;
                m_feed.setTitle( newUrl + '_' + m_feed.remoteId() );
                m_feed.setXmlUrl( newUrl );
                m_fetchTries++;
                doStart();
                return;
        }

        if ( status == Syndication::Success ) {
                // set the corresponding collection attributes
                m_feed.setTitle( feedptr->title() );
                m_feed.setHtmlUrl( feedptr->link() );
                m_feed.setFeedType( "rss" );
                m_feed.setDescription( feedptr->description() );
        }

        m_fetchTries = 0;

        // commit the new collection
        Akonadi::CollectionCreateJob *job = new Akonadi::CollectionCreateJob( m_feed );
        connect( job, SIGNAL( result( KJob* ) ), this, SLOT( collectionCreationDone( KJob* ) ) );
}

void AddFeedJob::collectionCreationDone( KJob *job )
{
        if ( job->error() ) {
                setError( CollectionCreationFailed );
                setErrorText( job->errorString() );
        }
        else {
                m_feed = static_cast<Akonadi::CollectionCreateJob*>( job )->collection();
        }

        emitResult();
}

#include "addfeedjob.moc"
