/*
    Copyright (C) 2008    Dmitry Ivanov <vonami@gmail.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "exportopmljob.h"

#include <KDebug>
#include <KUrl>
#include <kio/job.h>
#include <kio/filejob.h>

#include <QtCore/QTimer>
#include <QtCore/QXmlStreamWriter>
#include <QtCore/QByteArray>
#include <QtCore/QIODevice>
#include <QtCore/QBuffer>

using namespace KRss;

class ExportOpmlJobPrivate
{
public:

    explicit ExportOpmlJobPrivate( const QString &filepath, const QList<FeedCollection> &feeds )
        : m_filepath( filepath ), m_feeds( feeds )
    {
    }

    const QString m_filepath;
    const QList<FeedCollection> m_feeds;
    QXmlStreamWriter m_writer;
};

ExportOpmlJob::ExportOpmlJob( const QString &filepath, const QList<FeedCollection> &feeds, QObject *parent )
    : KJob( parent ), d( new ExportOpmlJobPrivate( filepath, feeds ) )
{
}

ExportOpmlJob::~ExportOpmlJob()
{
    delete d;
}

void ExportOpmlJob::start()
{
    QTimer::singleShot( 0, this, SLOT( doStart() ) );
}

QString ExportOpmlJob::errorString() const
{
    switch ( error() ) {
        case OpmlWriteFailed:
            return errorText();
        default:
            return KJob::errorString();
    }

    return QString();
}

void ExportOpmlJob::doStart()
{
    KIO::FileJob *job = KIO::open( KUrl( d->m_filepath ), QIODevice::WriteOnly );
    connect( job, SIGNAL( result( KJob* ) ), this, SLOT( slotResult( KJob*) ) );
    connect( job, SIGNAL( open( KIO::Job* ) ), this, SLOT( slotOpmlOpen( KIO::Job*) ) );
    connect( job, SIGNAL( written( KIO::Job*, KIO::filesize_t ) ), this, SLOT( slotWritten( KIO::Job*, KIO::filesize_t ) ) );
    job->start();
}

void ExportOpmlJob::slotResult( KJob *job )
{
    if ( job->error() ) {
        kDebug() << "Error:" << job->errorString();
    }
}

void ExportOpmlJob::slotOpmlOpen( KIO::Job *job )
{
    kDebug() << "OPML open, ready for writing";
    KIO::FileJob *fileJob = static_cast<KIO::FileJob *>( job );

    QBuffer buffer;
    buffer.open( QIODevice::WriteOnly );
    d->m_writer.setDevice( &buffer );

    d->m_writer.writeStartDocument();
    d->m_writer.writeStartElement( "opml" );
    d->m_writer.writeAttribute( "version", "2.0" );
    d->m_writer.writeEmptyElement( "head" );
    d->m_writer.writeStartElement( "body" );
    Q_FOREACH( const FeedCollection &feed, d->m_feeds ) {
        if ( feed.parent() != Akonadi::Collection::root().id() ) {
            writeOutline( feed );
        }
    }
    d->m_writer.writeEndElement(); // body
    d->m_writer.writeEndElement(); // opml
    d->m_writer.writeEndDocument();

    buffer.close();
    kDebug() << "Size:" << buffer.data().size();
    kDebug() << "Writing:" << buffer.data();
    fileJob->write( buffer.data() );
}

void ExportOpmlJob::slotWritten( KIO::Job *job, KIO::filesize_t written )
{
    kDebug() << "Written:" << written;
    static_cast<KIO::FileJob *>( job )->close();
}

void ExportOpmlJob::writeOutline( const FeedCollection &feed )
{
    Q_ASSERT( feed.hasAttribute( "FeedProperties" ) );

    d->m_writer.writeEmptyElement( "outline" );
    d->m_writer.writeAttribute( "id", feed.remoteId() );
    d->m_writer.writeAttribute( "text", feed.title() );
    d->m_writer.writeAttribute( "title", feed.title() );
    d->m_writer.writeAttribute( "description", feed.description() );
    d->m_writer.writeAttribute( "htmlUrl", feed.htmlUrl() );
    d->m_writer.writeAttribute( "xmlUrl", feed.xmlUrl() );

    // TODO(dmitry) proper tag handling
    /*if ( !feed.tags().isEmpty() ) {
        const QStringList tags = feed.tags();
        QString category;
        Q_FOREACH( const QString &tag, tags ) {
            category += tag + ',';
        }
        category.remove( category.length() - 1, 1 );
        d->m_writer.writeAttribute( "category", category );
    }*/
}

#include "exportopmljob.moc"
