/*
    Copyright (C) 2008    Dmitry Ivanov <vonami@gmail.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "importopmljob.h"

#include <KDebug>
#include <KUrl>
#include <KRandom>
#include <kio/job.h>
#include <kio/filejob.h>
#include <akonadi/collectioncreatejob.h>

#include <QtCore/QTimer>
#include <QtCore/QXmlStreamReader>
#include <QtCore/QByteArray>
#include <QtCore/QIODevice>

using namespace KRss;

// helper functions
static QStringRef attributeValue( const QXmlStreamAttributes &attributes, const QString &name )
{
    Q_FOREACH( const QXmlStreamAttribute &attr, attributes ) {
        if ( attr.name().toString().toLower() == name ) {
            return attr.value();
        }
    }

    return QStringRef();
}

class ImportOpmlJobPrivate
{
public:

    explicit ImportOpmlJobPrivate( const QString &filepath, const FeedCollection &rootCollection )
        : m_filepath( filepath ), m_rootCollection( rootCollection ), m_pendingJobs( 0 )
    {
    }

    const QString m_filepath;
    FeedCollection m_rootCollection;
    QString m_defaultTag;
    QXmlStreamReader m_reader;
    int m_pendingJobs;
    QList<FeedCollection> m_pendingFeeds;
    QHash<FeedCollection::Id, FeedCollection> m_feeds;
};

ImportOpmlJob::ImportOpmlJob( const QString &filepath, const FeedCollection &rootCollection, QObject *parent )
    : KJob( parent ), d( new ImportOpmlJobPrivate( filepath, rootCollection ) )
{
}

ImportOpmlJob::~ImportOpmlJob()
{
    delete d;
}

void ImportOpmlJob::setDefaultTag( const QString &defaultTag )
{
    d->m_defaultTag = defaultTag;
}

QHash<FeedCollection::Id, FeedCollection> ImportOpmlJob::importedFeeds() const
{
    return d->m_feeds;
}

FeedCollection ImportOpmlJob::importedRootCollection() const
{
    return d->m_rootCollection;
}

void ImportOpmlJob::start()
{
    QTimer::singleShot( 0, this, SLOT( doStart() ) );
}

QString ImportOpmlJob::errorString() const
{
    switch ( error() ) {
        case RootCollectionImportFailed:
        case OpmlReadFailed:
        case OpmlParsingFailed:
        case FeedImportFailed:
            return errorText();
        default:
            return KJob::errorString();
    }

    return QString();
}

void ImportOpmlJob::doStart()
{
    // start with creating the root collection
    if ( !d->m_rootCollection.isValid() ) {
        kDebug() << "Creating root collection";
        FeedCollection root;
        root.setParent( FeedCollection::root() );
        root.setRemoteId( KRandom::randomString( 10 ) );
        root.setName( "RSS subscriptions_" + root.remoteId() );
        root.setContentMimeTypes( QStringList( FeedCollection::mimeType() ) );

        Akonadi::CollectionCreateJob *job = new Akonadi::CollectionCreateJob( root );
        connect( job, SIGNAL( result( KJob* ) ), this, SLOT( slotRootCollectionImportDone( KJob* ) ) );
    }
    else {
        KIO::FileJob *job = KIO::open( KUrl( d->m_filepath ), QIODevice::ReadOnly );
        connect( job, SIGNAL( result( KJob* ) ), this, SLOT( slotFileResult( KJob* ) ) );
        connect( job, SIGNAL( open( KIO::Job* ) ), this, SLOT( slotOpmlOpen( KIO::Job*) ) );
        connect( job, SIGNAL( data( KIO::Job*, const QByteArray& ) ), this, SLOT( slotData( KIO::Job*, const QByteArray& ) ) );
        job->start();
    }
}

void ImportOpmlJob::slotRootCollectionImportDone( KJob *job )
{
    if ( job->error() ) {
        setError( RootCollectionImportFailed );
        setErrorText( job->errorString() );
        emitResult();
    }
    else {
        d->m_rootCollection = static_cast<Akonadi::CollectionCreateJob *>( job )->collection();
        kDebug() << "Root collection created, id:" << d->m_rootCollection.id();

        KIO::FileJob *job = KIO::open( KUrl( d->m_filepath ), QIODevice::ReadOnly );
        connect( job, SIGNAL( result( KJob* ) ), this, SLOT( slotFileResult( KJob* ) ) );
        connect( job, SIGNAL( open( KIO::Job* ) ), this, SLOT( slotOpmlOpen( KIO::Job*) ) );
        connect( job, SIGNAL( data( KIO::Job*, const QByteArray& ) ), this, SLOT( slotData( KIO::Job*, const QByteArray& ) ) );
        job->start();
    }
}

void ImportOpmlJob::slotOpmlOpen( KIO::Job *job )
{
    KIO::FileJob *fileJob = static_cast<KIO::FileJob *>( job );
    fileJob->read( fileJob->size() );
}

void ImportOpmlJob::slotData( KIO::Job *job, const QByteArray &data )
{
    kDebug() << "OPML read, ready for parsing";
    d->m_reader.addData( data );

    while ( !d->m_reader.atEnd() ) {
        d->m_reader.readNext();

        if ( d->m_reader.isStartElement() ) {
            if ( d->m_reader.name().toString().toLower() == "opml" ) {
                kDebug() << "OPML version" << d->m_reader.attributes().value( "version" ).toString();
                readOpml();
            }
            else {
                d->m_reader.raiseError( "The file is not an OPML document." );
            }
        }
    }

    static_cast<KIO::FileJob *>( job )->close();

    // commit the feeds or return the error from the parser
    if ( !d->m_reader.hasError() ) {
        Q_FOREACH( const FeedCollection &feed, d->m_pendingFeeds ) {
            kDebug() << "Importing new feed:" << feed.title();
            d->m_pendingJobs++;
            Akonadi::CollectionCreateJob *job = new Akonadi::CollectionCreateJob( feed );
            connect( job, SIGNAL( result( KJob* ) ), this, SLOT( slotFeedImportDone( KJob* ) ) );
        }
        d->m_pendingFeeds.clear();
    }
    else {
        setError( OpmlParsingFailed );
        setErrorText( d->m_reader.errorString() );
        emitResult();
    }
}

void ImportOpmlJob::slotFileResult( KJob *job )
{
    if ( job->error() ) {
        setError( OpmlReadFailed );
        setErrorText( job->errorString() );
        emitResult();
    }

    // otherwise, closed succesfully
}

void ImportOpmlJob::slotFeedImportDone( KJob *job )
{
    kDebug() << "Feed import done";
    if ( job->error() ) {
        setError( FeedImportFailed );
        setErrorText( job->errorString() );
    }
    else {
        FeedCollection feed = static_cast<Akonadi::CollectionCreateJob*>( job )->collection();
        d->m_feeds[ feed.id() ] = feed;
    }

    d->m_pendingJobs--;
    checkDone();
}

void ImportOpmlJob::checkDone()
{
    if ( d->m_pendingJobs > 0 )
        return;

    emitResult();
}

void ImportOpmlJob::readUnknownElement()
{
    Q_ASSERT( d->m_reader.isStartElement() );

    while ( !d->m_reader.atEnd() ) {
        d->m_reader.readNext();

        if ( d->m_reader.isEndElement() )
            break;

        if ( d->m_reader.isStartElement() )
            readUnknownElement();
    }
}

void ImportOpmlJob::readOpml()
{
    Q_ASSERT( d->m_reader.isStartElement() && d->m_reader.name().toString().toLower() == "opml" );

    while ( !d->m_reader.atEnd() ) {
        d->m_reader.readNext();

        if ( d->m_reader.isEndElement() )
            break;

        if ( d->m_reader.isStartElement() ) {
            if ( d->m_reader.name().toString().toLower() == "head" ) {
                readUnknownElement();
            }
            else if ( d->m_reader.name().toString().toLower() == "body" ) {
                readBody();
            }
            else {
                readUnknownElement();
            }
        }
    }
}

void ImportOpmlJob::readBody()
{
    Q_ASSERT( d->m_reader.isStartElement() && d->m_reader.name().toString().toLower() == "body" );

    while ( !d->m_reader.atEnd() ) {
        d->m_reader.readNext();

        if ( d->m_reader.isEndElement() )
            break;

        if ( d->m_reader.isStartElement() ) {
            if ( d->m_reader.name().toString().toLower() == "outline" ) {
                QStringList tags;
                if ( !d->m_defaultTag.isEmpty() ) {
                    tags << d->m_defaultTag;
                }
                readOutline( tags );
            }
            else {
                readUnknownElement();
            }
        }
    }
}

void ImportOpmlJob::readOutline( QStringList &tags )
{
    Q_ASSERT( d->m_reader.isStartElement() && d->m_reader.name().toString().toLower() == "outline" );

    bool isFolder = false;
    QString xmlUrl = attributeValue( d->m_reader.attributes(), "xmlurl" ).toString();

    if ( xmlUrl.isEmpty() ) {
        if ( !attributeValue( d->m_reader.attributes(), "text" ).isEmpty() ) {
            // this is an Akregator's folder
            isFolder = true;
            if ( !tags.contains( attributeValue( d->m_reader.attributes(), "text" ).toString() ) )
                tags.append( attributeValue( d->m_reader.attributes(), "text" ).toString() );

            kDebug() << "Tags:" << tags;
        }
        else {
            kDebug() << "Encountered an empty outline";
            // TODO: dump the rest of the block
        }
    }
    else {
        // this is a feed
        kDebug() << "Feed:" << xmlUrl;
        isFolder = false;
        FeedCollection feed;
        feed.setRemoteId( KRandom::randomString( 10 ) );
        feed.setName( xmlUrl + '_' + feed.remoteId() );
        feed.setParent( d->m_rootCollection );
        feed.setContentMimeTypes( QStringList( "application/rss+xml" ) );

        // set feed properties
        feed.setTitle( attributeValue( d->m_reader.attributes(), "title" ).toString() );
        if ( feed.title().isEmpty() ) {
            feed.setTitle( attributeValue( d->m_reader.attributes(), "text" ).toString() );
        }
        if ( feed.title().isEmpty() ) {
            feed.setTitle( xmlUrl );
        }
        feed.setXmlUrl( xmlUrl );
        feed.setHtmlUrl( attributeValue( d->m_reader.attributes(), "htmlurl" ).toString() );
        feed.setDescription( attributeValue( d->m_reader.attributes(), "description" ).toString() );
        feed.setFeedType( attributeValue( d->m_reader.attributes(), "type" ).toString() );
        if ( feed.feedType().isEmpty() ) {
            feed.setFeedType( "rss" );
        }

        // add tags from the 'category' attribute (OPML v2)
        if ( !attributeValue( d->m_reader.attributes(), "category" ).isEmpty() ) {
            QStringList categories = attributeValue( d->m_reader.attributes(), "category" ).toString().
                                                     split( QRegExp( "[,/]" ), QString::SkipEmptyParts );
            Q_FOREACH( const QString &category, categories ) {
                if ( !tags.contains( category ) )
                    tags.append( category );
            }
        }

        // set feed tags
        kDebug() << "Setting tags:" << tags;
        // TODO(dmitry) proper tag handling
        //feed.setTags( tags );

        d->m_pendingFeeds.append( feed );
    }

    while ( !d->m_reader.atEnd() ) {
        d->m_reader.readNext();

        if ( d->m_reader.isEndElement() )
            break;

        if ( d->m_reader.isStartElement() ) {
            if ( d->m_reader.name().toString().toLower() == "outline" && isFolder )
                readOutline( tags );
            else
                readUnknownElement();
        }
    }

    // once we are back from recursion remove the added tag
    // from the top of the list
    if ( isFolder )
        tags.removeLast();
}

#include "importopmljob.moc"
