/*
    Copyright (C) 2008    Dmitry Ivanov <vonami@gmail.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "importopmljobtest.h"
#include "importopmljob.h"

#include "libkrss/resourcemanager.h"

#include <akonadi/collectionfetchjob.h>
#include <akonadi/collectiondeletejob.h>
#include <qtest_kde.h>

using namespace KRss;

QTEST_KDEMAIN( ImportOpmlJobTest, NoGUI )

// list compare which ignores the order (taken from kdepimlibs/akonadi/tests)
template <class T> static void compareLists( const QList<T> &l1, const QList<T> &l2 )
{
    QCOMPARE( l1.count(), l2.count() );
    Q_FOREACH ( const T entry, l1 ) {
        QVERIFY( l2.contains( entry ) );
    }
}

void ImportOpmlJobTest::initTestCase()
{
    ResourceManager::registerAttributes();

    Akonadi::CollectionFetchJob *fetch = new Akonadi::CollectionFetchJob( Akonadi::Collection( 2 ),
                                                                          Akonadi::CollectionFetchJob::Recursive );

    if ( !fetch->exec() ) {
        qDebug() << "Fetch failed:" << fetch->errorString();
        return;
    }

    Q_FOREACH( const Akonadi::Collection &col, fetch->collections() ) {
        qDebug() << "Id:" << col.id() << " , name:" << col.name();
        Akonadi::CollectionDeleteJob *del = new Akonadi::CollectionDeleteJob( col );
        if ( !del->exec() ) {
            qDebug() << "Delete failed:" << fetch->errorString();
        }
    }
}

void ImportOpmlJobTest::testGoodOpml()
{
    Akonadi::Collection root( 2 );
    const QString defaultTag = "Imported";

    qDebug() << "Test data:" << TESTDATAPATH;
    ImportOpmlJob *import = new ImportOpmlJob( QString( "%1/good.opml" ).arg( TESTDATAPATH ), root, this );
    import->setDefaultTag( defaultTag );
    QVERIFY( import->exec() );

    // now walk through the collections and verify
    // that every property is set correctly
    QList<FeedCollection> feeds = import->importedFeeds().values();
    QVERIFY( feeds.count() == 5 );

    Q_FOREACH( const FeedCollection &feed, feeds ) {
        qDebug() << "Xml:" << feed.xmlUrl();
        // basic test
        if ( feed.xmlUrl() == "http://planetkde.org/rss20.xml" ) {
            QCOMPARE( feed.title(), QString( "Planet KDE" ) );
            QCOMPARE( feed.htmlUrl(), QString( "http://planetKDE.org/" ) );
            QCOMPARE( feed.description(), QString( "Planet KDE - http://planetKDE.org/" ) );
            QCOMPARE( feed.feedType(), QString( "rss" ) );
            // TODO(dmitry): proper tag handling
            //compareLists( feed.tags(), QStringList() << "Imported" << "Linux" << "Planets" );
        }
        // verifies that 'category=Linux,Ubuntu' is parsed correctly and
        // tag 'Linux' is set only once
        else if ( feed.xmlUrl() == "http://planet.ubuntu.com/rss20.xml" ) {
            QCOMPARE( feed.title(), QString( "Planet Ubuntu" ) );
            QCOMPARE( feed.htmlUrl(), QString( "http://planet.ubuntu.com/" ) );
            QCOMPARE( feed.description(), QString( "Planet Ubuntu - http://planet.ubuntu.com/" ) );
            QCOMPARE( feed.feedType(), QString( "rss" ) );
            // TODO(dmitry): proper tag handling
            //compareLists( feed.tags(), QStringList() << "Imported" << "Linux" << "Planets" << "Ubuntu" );
        }
        // this time without any tags except the defaultTag
        else if ( feed.xmlUrl() == "http://www.jwz.org/cheesegrater/RSS/apod.rss" ) {
            QCOMPARE( feed.title(), QString( "Astronomy Picture of the Day" ) );
            QCOMPARE( feed.htmlUrl(), QString( "http://antwrp.gsfc.nasa.gov/apod/" ) );
            QCOMPARE( feed.description(), QString( "A different image or photograph of our fascinating universe" ) );
            QCOMPARE( feed.feedType(), QString( "rss" ) );
            // TODO(dmitry): proper tag handling
            //compareLists( feed.tags(), QStringList() << "Imported" );
        }
        // verify that category="/Linux/News,/Technology" is parsed correctly
        else if ( feed.xmlUrl() == "http://lwn.net/headlines/newrss" ) {
            QCOMPARE( feed.title(), QString( "LWN.net" ) );
            QCOMPARE( feed.htmlUrl(), QString( "http://lwn.net" ) );
            QCOMPARE( feed.description(), QString( "LWN.net is a comprehensive source of news and opinions from and about the Linux community" ) );
            QCOMPARE( feed.feedType(), QString( "rss" ) );
            // TODO(dmitry): proper tag handling
            //compareLists( feed.tags(), QStringList() << "Imported" << "Linux" << "News" << "Technology" );
        }
        // make sure that a single tag in category=/News is parsed correctly
        else if ( feed.xmlUrl() == "http://newsrss.bbc.co.uk/rss/newsonline_world_edition/front_page/rss.xml" ) {
            QCOMPARE( feed.title(), QString( "BBC News | News Front Page | World Edition" ) );
            QCOMPARE( feed.htmlUrl(), QString( "http://news.bbc.co.uk/go/rss/-/2/hi/default.stm" ) );
            QCOMPARE( feed.description(), QString( "Visit BBC News for up-to-the-minute news" ) );
            QCOMPARE( feed.feedType(), QString( "rss" ) );
            // TODO(dmitry): proper tag handling
            //compareLists( feed.tags(), QStringList() << "Imported" << "News" );
        }
        else {
            QFAIL( "I didn't expect to see this feed in the test data" );
        }
    }
}

void ImportOpmlJobTest::testBrokenOpml()
{
    Akonadi::Collection root( 2 );
    const QString defaultTag = "Imported";

    qDebug() << "Test data:" << TESTDATAPATH;
    ImportOpmlJob *import = new ImportOpmlJob( QString( "%1/broken.opml" ).arg( TESTDATAPATH ), root, this );
    import->setDefaultTag( defaultTag );
    QVERIFY( !import->exec() );
    QVERIFY( import->error() == ImportOpmlJob::OpmlParsingFailed );
    qDebug() << "The error is:" << import->errorString();
}

void ImportOpmlJobTest::cleanupTestCase()
{
    Akonadi::CollectionFetchJob *fetch = new Akonadi::CollectionFetchJob( Akonadi::Collection( 2 ),
                                                                          Akonadi::CollectionFetchJob::Recursive );

    if ( !fetch->exec() ) {
        qDebug() << "Fetch failed:" << fetch->errorString();
        return;
    }

    Q_FOREACH( const Akonadi::Collection &col, fetch->collections() ) {
        qDebug() << "Id:" << col.id() << " , name:" << col.name();
        Akonadi::CollectionDeleteJob *del = new Akonadi::CollectionDeleteJob( col );
        if ( !del->exec() ) {
            qDebug() << "Delete failed:" << fetch->errorString();
        }
    }
}

#include "importopmljobtest.moc"
